﻿# -*- coding: utf-8 -*-
"""
qlearning.pyプログラム
強化学習(Q学習)の例題プログラム  
迷路の探索を学習します
使い方　c:\>python qlearning.py
"""
# モジュールのインポート
import math
import random

# グローバル変数
GENMAX = 1000   #学習の繰り返し回数
NODENO = 15     #Ｑ値のノード数
ALPHA = 0.1     #学習係数
GAMMA = 0.9     #割引率
EPSILON = 0.3   #行動選択のランダム性を決定
SEED = 32767    #乱数のシード

# 下請け関数の定義
# selecta()関数
def selecta(olds,qvalue):
    """行動を選択する"""
    #ε-greedy法による行動選択
    if random.random()<EPSILON:
        #ランダムに行動
        if(random.randint(0,1) == 0):
             s = 2 * olds + 1
        else:
             s = 2 * olds + 2
    else: 
        #Ｑ値最大値を選択
        if (qvalue[2 * olds + 1]) > (qvalue[2 * olds + 2]):
            s = 2 * olds + 1
        else:
            s = 2 * olds + 2
    return  s
# selecta()関数の終わり

# updateq()関数
def updateq(s,qvalue):
    """Q値を更新する"""
    if(s > 6):
        #最下段の場合
        if s == 14:
            #報酬の付与
            qv = qvalue[s] + int(ALPHA * (1000 - qvalue[s]))
            #報酬を与えるノードを増やす
            #他のノードを追加する場合は
            #下記2行のコメントを外す
#        elif s == 11:
#            qv = qvalue[s] + int(ALPHA * (500 - qvalue[s]))
        else:
            #報酬なし
            qv = qvalue[s]
    #最下段以外
    else:
        if (qvalue[2 * s + 1])>(qvalue[2 * s + 2]):
            qmax = qvalue[2 * s + 1]
        else:
            qmax = qvalue[2 * s +2 ]
        qv = qvalue[s] + int(ALPHA * (GAMMA * qmax - qvalue[s])) 
    return  qv
# updateq()関数の終わり

# メイン実行部
qvalue = [0 for i in range(NODENO)]      #Q値

#乱数の初期化
random.seed(SEED)

#Ｑ値の初期化
for i in range(NODENO):
    qvalue[i] = random.randint(0,100) 
print(qvalue)

#学習の本体
for i in range(GENMAX):
    s = 0   #行動の初期状態
    #最下段まで繰り返す
    for t in range(3):
        #行動選択
        s = selecta(s,qvalue)
        #Q値の更新
        qvalue[s] = updateq(s,qvalue)
    #Q値の出力
    print(qvalue)
    
# qlearning.pyの終わり
