﻿# -*- coding: utf-8 -*-
"""
direct.pyプログラム
全探索でナップサック問題を解く
使い方　c:\>python direct.py < data.txt
"""
# モジュールのインポート
import math
import copy

# グローバル変数
MAXVALUE = 100                  #重さと価値の最大値
N = 30                          #荷物の個数
WEIGHTLIMIT = N * MAXVALUE / 4  #重量制限
SEED = 32767                    #乱数のシード

# 下請け関数の定義
# initparcel()関数
def initparcel(percel):
    """荷物の初期化"""
    i = 0
    while i < N:
        try:
            line = input()
        except EOFError:
            break #入力の終わり
        parcel[i] = [int(num) for num in line.split()]
        i += 1
    return 
# initparcel()関数の終わり

# solve()関数
def solve(parcel):
    """探索の本体"""
    maxvalue = 0   #評価値の最大値
    #探索範囲の設定
    limit = pow(2,N)
    #解の探索
    for i in range(limit):
        #評価値の計算
        value = calcval(parcel,i)
        #最大値の更新
        if value > maxvalue:
            maxvalue = value
            solution = i
            print("*** maxvalue ",maxvalue)
    return solution
# solve()関数の終わり

# calcval()関数
def calcval(parcel,i):
    """適応度の計算"""
    value = 0         #評価値
    weight = 0        #重量
    #各遺伝子座を調べて重量と評価値を計算
    for pos in range(N):
        weight += parcel[pos][0] * ((i >> pos) & 0b1)
        if weight >= WEIGHTLIMIT:#致死遺伝子
            break 
    #致死遺伝子の処理
    if weight >= WEIGHTLIMIT:
        value = 0
    else:
        for pos in range(N):
            value  += parcel[pos][1] * ((i >> pos) & 0b1)
    return value
# calcval()関数の終わり

# メイン実行部
parcel = [[0 for i in range(2)] 
         for j in range(N)]     #荷物

#荷物の初期化
initparcel(parcel)

#探索の本体
solution = solve(parcel)

#解の出力
print(format(solution,'030b'))
# direct.pyの終わり
