﻿# -*- coding: utf-8 -*-
"""
cp.pyプログラム
畳み込みとプーリングの処理  
2次元データを読み取り、畳み込みとプーリングを施します
使い方　c:\>python cp.py < data1.txt
"""
# モジュールのインポート
import math
import sys

# グローバル変数
INPUTSIZE = 11    #入力数
FILTERSIZE = 3    #フィルタの大きさ
POOLSIZE = 3      #プーリングサイズ
POOLOUTSIZE = 3   #プーリングの出力サイズ

# 下請け関数の定義
# getdata()関数
def getdata(e):
    """学習データの読み込み"""
    n_of_e = 0   #データセットの行数
    # データの入力
    for line in sys.stdin : 
        e[n_of_e] = [float(num) for num in line.split()]
        n_of_e += 1
    return 
# getdata()関数の終わり

# conv()関数
def conv(filter,e,convout):
    """畳み込みの計算"""
    startpoint = math.floor(FILTERSIZE / 2)  #畳み込み範囲の下限
    endpoint = INPUTSIZE - startpoint        #畳み込み範囲の上限
    for i in range(startpoint,endpoint):
            for j in range(startpoint,endpoint):
                convout[i][j] = calcconv(filter,e,i,j)
    return
# conv()関数の終わり

# calcconv()関数
def calcconv(filter,e,i,j):
    """フィルタの適用"""
    r = math.floor(FILTERSIZE / 2)  #繰返しの範囲
    sum = 0.0
    for m in range(FILTERSIZE):
            for n in range(FILTERSIZE):
                sum += e[i - r + m][j - r + n] * filter[m][n]
    return sum
# calcconv()関数の終わり

# pool()関数
def pool(convout,poolout):
    """プーリングの計算"""
    for i in range(POOLOUTSIZE):
        for j in range(POOLOUTSIZE):
            poolout[i][j] = maxpooling(convout,i,j)
    return 
# pool()関数の終わり

# maxpooling()関数
def maxpooling(convout,i,j):
    """最大値プーリング"""
    #値の設定
    h = math.floor(POOLSIZE / 2)    #プーリングのサイズの1/2
    max = convout[i * POOLOUTSIZE + 1+ h][j * POOLOUTSIZE + 1 + h]
    mstart = POOLOUTSIZE * i + 1                  #mの開始値
    mend = POOLOUTSIZE * i + 1 + POOLSIZE - h + 1 #mの終了値
    nstart = POOLOUTSIZE * j + 1                  #nの開始値
    nend = POOLOUTSIZE * j + 1 + POOLSIZE - h + 1 #nの終了値
    #最大値の検出
    for m in range(mstart,mend):
        for n in range(nstart,nend):
            if max < convout[m][n]:
                max=convout[m][n]
    return max
# maxpooling()関数の終わり

# メイン実行部
#filter = [[0,0,0],[1,1,1],[0,0,0]]          #横フィルタ
filter = [[0,1,0],[0,1,0],[0,1,0]]           #縦フィルタ
e = [[0.0 for i in range(INPUTSIZE)] 
      for j in range(INPUTSIZE)]             #入力データ
convout = [[0.0 for i in range(INPUTSIZE)] 
            for j in range(INPUTSIZE)]       #畳み込み出力
poolout = [[0.0 for i in range(POOLOUTSIZE)] 
            for j in range(POOLOUTSIZE)]     #出力データ

#入力データの読み込み
getdata(e)

#畳み込みの計算
conv(filter,e,convout)
for i in convout:
    print(i)

#プーリングの計算
pool(convout,poolout)

#結果の出力
for i in poolout:
    print(i)
   
# cp.pyの終わり
