﻿# -*- coding: utf-8 -*-
"""
simplecnn.pyプログラム
畳み込みニューラルネットの基本構造デモンストレーション  
CNNの基本構造(骨組のみ）を示します
使い方　c:\>python simplecnn.py < data11.txt
"""
# モジュールのインポート
import math
import sys
import random

# グローバル変数
INPUTSIZE = 11    #入力数
FILTERSIZE = 3    #フィルタの大きさ
FILTERNO = 2      #フィルタの個数
POOLSIZE = 3      #プーリングサイズ
POOLOUTSIZE = 3   #プーリングの出力サイズ
HIDDENNO = 3      #中間層のセル数
ALPHA = 10        #学習係数
SEED = 65535      #乱数のシード
MAXINPUTNO = 100  #データの最大個数
BIGNUM = 100.0    #誤差の初期値
LIMIT = 0.001     #誤差の上限値

# 下請け関数の定義
# getdata()関数
def getdata(e,t):
    """入力データの読み込み"""
    n_of_e = -1   #データセットの個数
    # データの入力
    for line in sys.stdin : 
        if len(line.split()) == 1 :#教師データ
            n_of_e += 1
            l = 0    #画像の行を先頭にセット
            t[n_of_e] = int(line)
        elif len(line.split()) > 1:#画像データ
            e[n_of_e][l] = [float(num) for num in line.split()]
            l += 1
    return n_of_e + 1
# getdata()関数の終わり

# showdata()関数
def showdata(e,t,n_of_e):
    """入力データの表示"""
    for no in range(n_of_e):
        #カテゴリの表示
        print("N=",no," category:",t[no])
        for i in e[no]:
            print(i)
    return 
# showdata()関数の終わり

# conv()関数
def conv(filter,e,convout):
    """畳み込みの計算"""
    startpoint = math.floor(FILTERSIZE / 2)  #畳み込み範囲の下限
    endpoint = INPUTSIZE - startpoint        #畳み込み範囲の上限
    for i in range(startpoint,endpoint):
            for j in range(startpoint,endpoint):
                convout[i][j] = calcconv(filter,e,i,j)
    return
# conv()関数の終わり

# calcconv()関数
def calcconv(filter,e,i,j):
    """フィルタの適用"""
    r = math.floor(FILTERSIZE / 2)  #繰返しの範囲
    sum = 0.0
    for m in range(FILTERSIZE):
            for n in range(FILTERSIZE):
                sum += e[i - r + m][j - r + n] * filter[m][n]
    return sum
# calcconv()関数の終わり

# pool()関数
def pool(convout,poolout):
    """プーリングの計算"""
    for i in range(POOLOUTSIZE):
        for j in range(POOLOUTSIZE):
            poolout[i][j] = maxpooling(convout,i,j)
    return 
# pool()関数の終わり

# maxpooling()関数
def maxpooling(convout,i,j):
    """最大値プーリング"""
    #値の設定
    h = math.floor(POOLSIZE / 2)    #プーリングのサイズの1/2
    max = convout[i * POOLOUTSIZE + 1+ h][j * POOLOUTSIZE + 1 + h]
    mstart = POOLOUTSIZE * i + 1                  #mの開始値
    mend = POOLOUTSIZE * i + 1 + POOLSIZE - h + 1 #mの終了値
    nstart = POOLOUTSIZE * j + 1                  #nの開始値
    nend = POOLOUTSIZE * j + 1 + POOLSIZE - h + 1 #nの終了値
    #最大値の検出
    for m in range(mstart,mend):
        for n in range(nstart,nend):
            if max < convout[m][n]:
                max=convout[m][n]
    return max
# maxpooling()関数の終わり

# forward()関数
def forward(wh,wo,hi,e):
    """順方向の計算"""
    #hiの計算
    for i in range(HIDDENNO):
        u = 0.0
        jlimit = POOLOUTSIZE * POOLOUTSIZE * FILTERNO
        for j in range(jlimit):
            u += e[j] * wh[i][j]
        u -= wh[i][jlimit] #しきい値の処理
        hi[i] = f(u)
    #出力oの計算
    o=0.0
    for i in range(HIDDENNO):
        o += hi[i] * wo[i]
    o -= wo[HIDDENNO] #しきい値の処理
    return f(o)
# forward()関数の終わり

# olearn()関数
def olearn(wo,hi,e,o):
    """出力層の重み学習"""
    #誤差の計算
    teacherno = POOLOUTSIZE * POOLOUTSIZE * FILTERNO
    d = (e[teacherno] - o) * o * (1 - o)
    #重みの学習
    for i in range(HIDDENNO):
        wo[i] += ALPHA * hi[i] * d
    #しきい値の学習
    wo[HIDDENNO] += ALPHA * (-1.0) * d
    return 
# olearn()関数の終わり

# hlearn()関数
def hlearn(wh,wo,hi,e,o):
    """中間層の重み学習"""
    #中間層の各セルjを対象
    for j in range(HIDDENNO):
        teacherno = POOLOUTSIZE * POOLOUTSIZE * FILTERNO
        dj = hi[j] * (1 - hi[j]) * wo[j] * (e[teacherno] - o) * o * (1 - o)
        #i番目の重みを処理
        for i in range(teacherno):
            wh[j][i] += ALPHA * e[i] * dj
        #しきい値の学習
        wh[j][teacherno] += ALPHA * (-1.0) * dj
    return 
# hlearn()関数の終わり

# f()関数
def f(u):
    """伝達関数"""
    #シグモイド関数の計算
    return 1.0/(1.0 + math.exp(-u))
# f()関数の終わり

# メイン実行部
#乱数の初期化
random.seed(SEED)

#変数の準備
e = [[[0.0 for i in range(INPUTSIZE)] 
       for j in range(INPUTSIZE)]
       for k in range(MAXINPUTNO)]           #入力データ
t = [0.0 for i in range(MAXINPUTNO)]         #教師データ
err = BIGNUM                                 #誤差の評価
convout = [[0.0 for i in range(INPUTSIZE)] 
            for j in range(INPUTSIZE)]       #畳み込み出力
poolout = [[0.0 for i in range(POOLOUTSIZE)] 
            for j in range(POOLOUTSIZE)]     #出力データ
ppf = POOLOUTSIZE * POOLOUTSIZE * FILTERNO   #efの初期化に利用
ef = [0.0 for i in range(ppf + 1)]           #全結合層への入力データ
hi = [0.0 for i in range(HIDDENNO+1)]        #中間層の出力
#フィルタの準備
filter = [[[random.uniform(-1,1) for i in range(FILTERSIZE)]
            for j in range(FILTERSIZE)]            
            for k in range(FILTERNO)]
#全結合層の準備
inputnumber = POOLOUTSIZE * POOLOUTSIZE * FILTERNO + 1
wh = [[random.uniform(-1,1) for i in range(inputnumber)]
       for j in range(HIDDENNO)]            #中間層の重み
wo = [random.uniform(-1,1)
      for i in range(HIDDENNO + 1)]         #出力層の重み

#入力データの読み込み
n_of_e = getdata(e,t)
showdata(e,t,n_of_e)

#学習
count = 0
while err > LIMIT:
    err = 0.0
    for i in range(n_of_e):#学習データ毎の繰り返し
        for j in range(FILTERNO):#フィルタ毎の繰り返し
            #畳み込みの計算
            conv(filter[j],e[i],convout)
            #プーリングの計算
            pool(convout,poolout)
            #プーリング出力を全結合層の入力へコピー
            for m in range(POOLOUTSIZE):
                for n in range(POOLOUTSIZE):
                    #全結合層の学習データ
                    ps = POOLOUTSIZE * POOLOUTSIZE
                    ef[j * ps + POOLOUTSIZE * m + n] = poolout[m][n]
                    #教師データ
                    ef[POOLOUTSIZE * POOLOUTSIZE * FILTERNO]=t[i]
        #順方向の計算
        o = forward(wh,wo,hi,ef)
        #出力層の重みの調整
        olearn(wo,hi,ef,o)
        #中間層の重みの調整
        hlearn(wh,wo,hi,ef,o)
        #誤差の積算
        err += (o - t[i]) * (o - t[i])
    count += 1
    #誤差の出力
    print(count," ",err) 
#結果出力
print("***Results***")
print("Weights")
#結合荷重の出力
print(wh,wo)
#学習データに対する出力 
print("Network output")
print("#   teacher    output")
for i in range(n_of_e):
    for j in range(FILTERNO):#フィルタ毎の繰り返し
        #畳み込みの計算
        conv(filter[j],e[i],convout)
        #プーリングの計算
        pool(convout,poolout)
        #プーリング出力を全結合層の入力へコピー
        for m in range(POOLOUTSIZE):
            for n in range(POOLOUTSIZE):
                #全結合層の学習データ
                ps = POOLOUTSIZE * POOLOUTSIZE
                ef[j * ps + POOLOUTSIZE * m + n] = poolout[m][n]
                #教師データ
                ef[POOLOUTSIZE * POOLOUTSIZE * FILTERNO]=t[i]
    #出力値の計算
    print(i,"     ",t[i],"    ",forward(wh,wo,hi,ef))   

# simplecnn.pyの終わり
